@@ -20,8 +20,8 @@ class WebPageTimeOutError(SCAPIError):
 
     def __init__(self, url):
         self.message = f"""
-            Impossibile raggiungere '{url}'.
-            Unable to reach '{url}'.
+            Impossibile raggiungere "{url}".
+            Unable to reach "{url}".
             """
         super().__init__(self.message)
 
@@ -42,8 +42,8 @@ class MatchNotFound(SCAPIError):
 
     def __init__(self, name):
         self.message = f"""
-            Impossibile estrarre {name}.
-            Unable to get {name}.
+            Impossibile estrarre "{name}".
+            Unable to get "{name}".
             """
         super().__init__(self.message)
 
@@ -53,8 +53,8 @@ class NoSeasonFoundError(SCAPIError):
 
     def __init__(self, name):
         self.message = f"""
-                Nessuna stagione trovata per la serie {name}
-                No Seasons Found for the series {name}
+                Nessuna stagione trovata per la serie "{name}".
+                No Seasons Found for the series "{name}".
                 """
         super().__init__(self.message)
 
@@ -64,8 +64,8 @@ class InvalidJSON(SCAPIError):
 
     def __init__(self, name, e, data):
         self.message = f"""
-            {name} contiene JSON non valido:
-            {name} contains Invalid JSON data:
+            "{name}" contiene JSON non valido:
+            "{name}" contains Invalid JSON data:
             Data: {data}
             Error: {e}
             """
@@ -77,11 +77,23 @@ class PreviewError(SCAPIError):
 
     def __init__(self, name, e):
         self.message = f"""
-            Impossibile ottenere i dati per {name}
-            Unable to get preview data for {name}
+            Impossibile ottenere i dati per "{name}".
+            Unable to get preview data for "{name}".
             Error: {e}
             """
         super().__init__(self.message)
+        
+class KeyNotFoundOrNone(SCAPIError):
+    """Raised when dict key not found or None"""
+
+    def __init__(self, key, name):
+        self.message = f"""
+            Impossibile estrarre "{key}" da:
+                {name}
+            Unable to get "{key}" from:
+                {name}.
+            """
+        super().__init__(self.message)
 
 
 class API:
@@ -173,15 +185,17 @@ class API:
         # Estrarre i risultati della ricerca
         # Extract the search results
         try:
-            search_results = document.json()["data"]
+            search_results = document.json().get("data")
             output_dict = {}
             for result in search_results:
-                result["url"] = f"{self._url}/titles/{result['id']}-{result['slug']}"
+                if (result.get("id") is None or result.get("slug") is None or result.get("name") is None):
+                    raise KeyNotFoundOrNone("id or slug", result)
+                result["url"] = f"{self._url.geturl()}/it/titles/{result['id']}-{result['slug']}"
                 output_dict[result["name"]] = result
         except Exception as e:
             raise InvalidJSON(query, e, document) from e
 
-        return output_dict  # [result for result in search_results]
+        return output_dict
 
     def preview(self, content_slug):
         """
@@ -197,7 +211,7 @@ class API:
             dict:
                 Un dizionario contenente informazioni minimali sull'elemento:
                 A dictionary containing detailed information about the item:
-                    {id, type, runtime, release_date, quality, plot, seasons_count, preview (only for movies), images, generes}.
+                    {id, type, runtime, release_date, quality, plot, seasons_count, preview (only for movies), images, genres}.
 
         Example:
         ```
@@ -240,7 +254,7 @@ class API:
         film_info = load('6203-movie-name')
         ```
         """
-        url = self._url.geturl() + "/titles/" + content_slug
+        url = self._url.geturl() + "/it/titles/" + content_slug
         try:
             # Ottenere la risposta dell'url dell'elemento
             # Get the response of the item's url
@@ -255,73 +269,89 @@ class API:
 
         # Estrarre i vari dati
         # Extract the various data
+        if (not preview_data.get("type")):
+            raise KeyNotFoundOrNone("type", preview_data)
         media_type = "Movie" if preview_data["type"] == "movie" else "TvSeries"
 
-        images = preview_data["images"]
+        images = preview_data.get("images")
 
-        year = preview_data["release_date"].split("-")[0]
+        year = preview_data["release_date"].split("-")[0] if preview_data["release_date"] else None
+        
+        if (data.get("props") is None or data["props"].get("title") is None):
+            raise KeyNotFoundOrNone("props or props.title", data)
 
         props = data["props"]
 
-        trailer_info = props["title"]["trailers"]
+        trailer_info = props["title"].get("trailers")
         trailer_url = (
             f"https://www.youtube.com/watch?v={trailer_info[0]['youtube_id']}"
-            if trailer_info
+            if trailer_info and trailer_info[0]["youtube_id"]
             else None
         )
-
-        correlates = props["sliders"][0]["titles"]
+        if props.get("sliders") is None or len(props["sliders"]) == 0 or props["sliders"][0].get("titles") is None:
+            correlates = []
+        else:
+            correlates = props["sliders"][0]["titles"]
+        
         size = min(len(correlates), 15)
         correlates_list = correlates[:size]
 
-        plot = props["title"]["plot"]
+        plot = props["title"].get("plot")
 
-        score = props["title"]["score"]
+        score = props["title"].get("score")
 
-        tmdb_id = props["title"]["tmdb_id"]
-        imdb_id = props["title"]["imdb_id"]
-        netflix_id = props["title"]["netflix_id"]
-        prime_id = props["title"]["prime_id"]
-        disney_id = props["title"]["disney_id"]
-        release_date = props["title"]["release_date"]
-        sub_ita = props["title"]["sub_ita"]
+        tmdb_id = props["title"].get("tmdb_id")
+        imdb_id = props["title"].get("imdb_id")
+        netflix_id = props["title"].get("netflix_id")
+        prime_id = props["title"].get("prime_id")
+        disney_id = props["title"].get("disney_id")
+        release_date = props["title"].get("release_date")
+        sub_ita = bool(props["title"]["sub_ita"]) if props["title"].get("sub_ita") else None
 
         # Estrarre i dati degli episodi per le serie
         # Extract episode data for series.
         if media_type == "TvSeries":
 
+            if (props["title"].get("name") is None):
+                raise KeyNotFoundOrNone("name", props["title"])
+
             name = props["title"]["name"]
 
-            seasons = props["title"]["seasons"]
+            seasons = props["title"]["seasons"] if props["title"]["seasons"] else []
 
-            seasons_count = int(props["title"]["seasons_count"])
+            seasons_count = int(props["title"]["seasons_count"]) if props["title"]["seasons_count"] else None
 
             episode_list = []
             for se in seasons:
+                if (se.get("number") is None or se.get("title_id") is None):
+                    raise KeyNotFoundOrNone("number or title_id", se)
                 season = int(se["number"])
-                se_url = f"{url}/stagione-{season}"
+                se_url = f"{url}/season-{season}"
                 try:
-                    resp = self._wbpage_as_text(url)
+                    resp = self._wbpage_as_text(se_url)
                     se_data = json.loads(
                         self._html_regex(r'data-page="([\s\S]+})"', resp, "page data")
                     )
                 except requests.exceptions.Timeout as e:
                     raise WebPageTimeOutError(se_url) from e
-                episodes = se_data["props"]["loadedSeason"]["episodes"]
+                if (se_data.get("props") is None or se_data["props"].get("loadedSeason") is None):
+                    raise KeyNotFoundOrNone("props or props.loadedSeason", se_data)
+                episodes = se_data["props"]["loadedSeason"]["episodes"] if se_data["props"]["loadedSeason"].get("episodes") else []
                 sid = se["title_id"]
                 for ep in episodes:
-                    scws_id = ep["scws_id"]
-                    href = f"{self._url}/watch/{sid}?e={ep['id']}"
+                    if (ep.get("id") is None):
+                        raise KeyNotFoundOrNone("id", ep)
+                    href = f"{self._url.geturl()}/it/watch/{sid}?e={ep['id']}"
 
                     episode = {
-                        "name": ep["name"],
+                        "name": ep.get("name"),
                         "season": season,
-                        "episode": int(ep["number"]),
-                        "description": ep["plot"],
-                        "duration": int(ep["duration"]),
-                        "images": ep["images"],
+                        "episode": int(ep["number"]) if ep.get("number") else None,
+                        "description": ep.get("plot"),
+                        "duration": int(ep["duration"]) if ep.get("duration") else None,
+                        "images": ep.get("images"),
                         "url": href,
-                        "scws_id": scws_id,
+                        "id": ep.get("id"),
                     }
                     episode_list.append(episode)
 
@@ -331,10 +361,11 @@ class API:
             return {
                 "name": name,
                 "url": url,
+                "id": props["title"].get("id"),
                 "type": media_type,
                 "episodeList": episode_list,
                 "images": images,
-                "year": int("".join(filter(str.isdigit, year))),
+                "year": int("".join(filter(str.isdigit, year))) if year else None,
                 "plot": plot,
                 "tmdb_id": tmdb_id,
                 "imdb_id": imdb_id,
@@ -342,21 +373,21 @@ class API:
                 "prime_id": prime_id,
                 "disney_id": disney_id,
                 "release_date": release_date,
-                "sub_ita": bool(sub_ita),
-                "rating": int(float(score) * 1000),
+                "sub_ita": sub_ita,
+                "rating": int(float(score) * 1000) if score else None,
                 "seasons_count": seasons_count,
-                "tags": [genre["name"] for genre in preview_data["genres"]],
+                "tags": [genre.get("name") for genre in preview_data["genres"]] if preview_data.get("genres") else None,
                 "trailerUrl": trailer_url,
                 "recommendations": correlates_list,
             }
 
         return {
-            "name": props["title"]["name"],
+            "name": props["title"].get("name"),
             "url": url,
-            "scws_id": props["title"]["scws_id"],
+            "id": props["title"].get("id"),
             "type": media_type,
             "images": images,
-            "year": int("".join(filter(str.isdigit, year))),
+            "year": int("".join(filter(str.isdigit, year))) if year else None,
             "plot": plot,
             "tmdb_id": tmdb_id,
             "imdb_id": imdb_id,
@@ -364,27 +395,30 @@ class API:
             "prime_id": prime_id,
             "disney_id": disney_id,
             "release_date": release_date,
-            "sub_ita": bool(sub_ita),
-            "rating": int(float(score) * 1000),
-            "tags": [genre["name"] for genre in preview_data["genres"]],
-            "duration": int(props["title"]["runtime"]),
+            "sub_ita": sub_ita,
+            "rating":  int(float(score) * 1000) if score else None,
+            "tags": [genre["name"] for genre in preview_data["genres"]] if preview_data.get("genres") else None,
+            "duration": int(props["title"]["runtime"]) if props["title"].get("runtime") else None,
             "trailerUrl": trailer_url,
             "recommendations": correlates_list,
         }
 
-    def get_links(self, content_id, episode_id=None, get_m3u=False):
+    def get_links(self, vixsrc_url, tmdb_id, tv=None, get_m3u=False):
         """
         Estrai la playlist m3u8
         Get the m3u8 playlist
 
         Args:
-            content_id (str | int):
-                L'ID dell'elemento.
-                The ID of the item.
+            vixsrc_url (str):
+                Url del server vixsrc
+                Url of vixsrc server
+            tmdb_id (int):
+                L'ID tmdb.
+                The tmdb ID.
 
-            episode_id (str | int | none):
-                L'ID dell'episodio se è una serie.
-                The ID of the episode if it's a series.
+            tv (tuple):
+                Una tupla contenente il numero di stagione e il numero dell'episodio.
+                A tuple containing the season number and episode number, if it's a series.
             get_m3u (bool):
                 Se si desidera direttamente il file m3u
                 If you want the m3u file
@@ -404,31 +438,10 @@ class API:
         headers = {
             "user-agent": self.user_agent,
         }
-
-        webpage = self._wbpage_as_text(
-            self._url.geturl()
-            + "/watch/"
-            + str(content_id)
-            + ("" if episode_id is None else ("&e=" + str(episode_id)))
-        )
-
-        # Extract information from data-page attribute
-        info = json.loads(
-            re.sub(
-                r',[^"]+}',
-                "}",
-                self._html_regex(r'data-page="([\s\S]+})"', webpage, "info"),
-            )
-        )
-
-        # Extract the video page url
-        video_page_url = self._wbpage_as_text(info["props"]["embedUrl"])
-
-        # Get the iframe url and iframe page
-        iframe_url = self._html_regex(
-            r'<iframe[^>]+src\s*=\s*"([^"]+)', video_page_url, "iframe url"
-        )
-        iframe_page = self._wbpage_as_text(iframe_url)
+        
+        vixsrc_iframe_url = f'https://{vixsrc_url}/{"tv" if tv else "movie"}/{tmdb_id}{"/" + str(tv[0]) + "/" + str(tv[1]) if tv else ""}'
+        
+        iframe_page = self._wbpage_as_text(vixsrc_iframe_url)
 
         # Extract the playlist params and url from the page js
         playlist_params = json.loads(
@@ -442,17 +455,7 @@ class API:
                 ).replace("'", '"'),
             )
         )
-        streams = json.loads(
-            self._html_regex(
-                r"window\.streams[^=]+=[^[](\[.*\])",
-                iframe_page,
-                "playlist streams",
-            )
-        )
 
-        # playlist_url = next((sub for sub in streams if sub["active"] is True), None)[
-        #     "url"
-        # ]
         playlist_url = self._html_regex(
             r"window\.masterPlaylist[^<]+url:[^<]+\'([^<]+?)\'",
             iframe_page,
@@ -467,7 +470,6 @@ class API:
             )
             == "true"
         )
-        # video_info = json.loads(self._html_regex(r'window\.video[^{]+({[^<]+});',vixcloud_iframe, "video info")
 
         # Generate the polaylist url
         dl_url = (
@@ -479,8 +481,6 @@ class API:
             + playlist_params.get("token")
             + ("&h=1" if can_play_fhd else "")
         )
-        
-        sc_iframe_url = f"{self._url.geturl()}/iframe/{content_id}"
 
         if get_m3u:
             m3u = None
@@ -497,6 +497,7 @@ class API:
             else:
                 raise WebPageStatusCodeError("m3u URL", m3u_response.status_code)
 
-            return iframe_url, dl_url, m3u
+            return vixsrc_iframe_url, dl_url, m3u
         else:
-            return iframe_url, dl_url
+            return vixsrc_iframe_url, dl_url
+
